// @flow
/**
 * Copyright (c) 2017-present, Evosphere.
 * All rights reserved.
 */

import Kernel from './core/Kernel';
import emptyObject from './util/emptyObject';

import type {
  KernelFactoryConfigurationType
} from './definitionTypes';

import type { KernelType } from './core/Kernel';
import tasksMixin from './tasksMixin';

/**
 * Creates a register function
 * @param kernel
 * @returns {function(string, Object): KernelType}
 */
function createRegisterFunction(kernel : KernelType) {
  return (name : string, service : Object) => kernel.set(name, service);
}

/**
 * Creates a Kernel
 * @param config
 * @returns {Promise.<Kernel>}
 */
KernelFactory.prototype.create = async function(config : KernelFactoryConfigurationType) : Promise<KernelType> {
  config = Object.assign({}, config);
  const env : string = typeof config.getEnvironment === "function" ? config.getEnvironment() : process.env.NODE_ENV || "production";
  const parameters = typeof config.getParameters === "function" ? config.getParameters(env) : emptyObject;

  // $FlowFixMe
  const kernel : KernelType = new Kernel();
  const register = createRegisterFunction(kernel);
  kernel.loadParameters(parameters);

  const services = this.executeTasksAndReturnValue("registerServices", [register, parameters, env]);
  services.forEach(obj => kernel.loadServices(obj));

  if(typeof config.registerServices === "function") {
    config.registerServices(register, parameters, env);
  }

  await this.executeTasks("configureKernel", [config, kernel]);
  return kernel;
};

/**
 * The Kernel is at the heart of the application. It manages the various services.
 * @constructor
 */
function KernelFactory(){}
tasksMixin(KernelFactory);

export default new KernelFactory();
