// @flow
/**
 * Copyright (c) 2017-present, Evosphere.
 * All rights reserved.
 */

export type BagType<$KeyType, $ValueType> = {[key: $KeyType] : $ValueType };

export type ContainerType = {
  set: (key : string, value : mixed) => void,
  get: (key : string) => mixed,
  all: () => BagType<string, mixed>,
};

/**
 * A container is an interface to access an encapsulated map.
 * @constructor
 */
function Container(){
  const bag: BagType<string, mixed> = {};

  this.set = function(key : string, value : mixed) : void {
    bag[key] = value;
  };
  this.get = function(key : string) : mixed {
    return bag[key];
  };
  this.all = function() : BagType<string, mixed> {
    return bag;
  }
}

export default {
  create() : ContainerType {
    return new Container();
  }
};